#############################################################################
#
# Copyright (c) 2006 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""catalog package test runner

$Id: tests.py 2918 2005-07-19 22:12:38Z jim $
"""

import unittest
from zope.testing import doctest, module
import zope.component.testing
import zope.component.factory
import zope.component.interfaces

import zc.catalog
import zc.catalog.interfaces

import BTrees.Interfaces
import BTrees.LOBTree
import BTrees.OLBTree
import BTrees.LFBTree


def setUp32bit(test):
    zope.component.testing.setUp(test)
    test.globs["btrees_family"] = BTrees.family32

def modSetUp32bit(test):
    setUp32bit(test)
    module.setUp(test, 'zc.catalog.doctest_test')

def setUp64bit(test):
    zope.component.testing.setUp(test)
    test.globs["btrees_family"] = BTrees.family64

def modSetUp64bit(test):
    setUp64bit(test)
    module.setUp(test, 'zc.catalog.doctest_test')

def tearDown(test):
    zope.component.testing.tearDown(test)

def modTearDown(test):
    module.tearDown(test)
    zope.component.testing.tearDown(test)


def test_suite():
    tests = unittest.TestSuite((
        # 32 bits
        doctest.DocFileSuite(
            'extentcatalog.txt', setUp=modSetUp32bit, tearDown=modTearDown,
            optionflags=doctest.INTERPRET_FOOTNOTES),
        doctest.DocFileSuite(
            'setindex.txt', setUp=setUp32bit, tearDown=tearDown,
            optionflags=doctest.INTERPRET_FOOTNOTES),
        doctest.DocFileSuite(
            'valueindex.txt', setUp=setUp32bit, tearDown=tearDown),
        doctest.DocFileSuite(
            'normalizedindex.txt', setUp=setUp32bit, tearDown=tearDown),
        doctest.DocFileSuite(
            'globber.txt', setUp=setUp32bit, tearDown=tearDown),
        doctest.DocFileSuite(
            'callablewrapper.txt', setUp=setUp32bit, tearDown=tearDown),

        # 64 bits
        doctest.DocFileSuite(
            'extentcatalog.txt', setUp=modSetUp64bit, tearDown=modTearDown,
            optionflags=doctest.INTERPRET_FOOTNOTES),
        doctest.DocFileSuite('setindex.txt', setUp=setUp64bit,
                             tearDown=tearDown),
        doctest.DocFileSuite('valueindex.txt', setUp=setUp64bit,
                             tearDown=tearDown),
        doctest.DocFileSuite('normalizedindex.txt', setUp=setUp64bit,
                             tearDown=tearDown),
        doctest.DocFileSuite('globber.txt', setUp=setUp64bit,
                             tearDown=tearDown),
        doctest.DocFileSuite('callablewrapper.txt', setUp=setUp64bit,
                             tearDown=tearDown),

        # legacy data support
        doctest.DocFileSuite('legacy.txt', optionflags=doctest.ELLIPSIS),
        ))
    import zc.catalog.stemmer
    if not zc.catalog.stemmer.broken:
        tests.addTest(doctest.DocFileSuite('stemmer.txt'))
    return tests

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



#############################################################################
#
# Copyright (c) 2006 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""A stemmer based on the textindexng stemmer, itself based on snowball.

$Id: stemmer.py 2918 2005-07-19 22:12:38Z jim $
"""
import re
broken = None
try:
    from zopyx.txng3 import stemmer
except ImportError:
    try:
        import txngstemmer as stemmer
    except ImportError:
        stemmer = None
        class Broken:
            def stem(self, l):
                return l
        broken = Broken()

# as of this writing, trying to persist a txngstemmer.Stemmer makes the python
# process end, only printing a "Bus error" message before quitting.  Don't do
# that. July 16 2005

class Stemmer(object):

    def __init__(self, language='english'):
        self.language = language

    @property
    def stemmer(self):
        if stemmer is None:
            return broken
        return stemmer.Stemmer(self.language)

    rxGlob = re.compile(r"[*?]") # See globToWordIds() in
    # zope/index/text/lexicon.py

    def process(self, lst):
        stemmer = self.stemmer
        result = []
        for s in lst:
            try:
                s = unicode(s)
            except UnicodeDecodeError:
                pass
            else:
                s = stemmer.stem((s,))[0]
            result.append(s)
        return result

    def processGlob(self, lst):
        stemmer = self.stemmer
        result = []
        rxGlob = self.rxGlob
        for s in lst:
            if not rxGlob.search(s):
                try:
                    s = unicode(s)
                except UnicodeDecodeError:
                    pass
                else:
                    s = stemmer.stem((s,))[0]
            result.append(s)
        return result



#############################################################################
#
# Copyright (c) 2006 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""glob all terms at the end.

$Id: globber.py 2918 2005-07-19 22:12:38Z jim $
"""
from zope.index.text import queryparser, parsetree

reconstitute = {}
reconstitute["NOT"] = lambda nd: "not %s" % (
    reconstitute[nd.getValue().nodeType()](nd.getValue()),)
reconstitute["AND"] = lambda nd: "(%s)" % (" and ".join(expand(nd)),)
reconstitute["OR"] = lambda nd: "(%s)" % (" or ".join(expand(nd)),)
reconstitute["ATOM"] = lambda nd: '%s*' % (nd.getValue())
reconstitute["PHRASE"] = lambda nd: '"%s"' % (
    ' '.join((v + '*') for v in nd.getValue()),)
reconstitute["GLOB"] = lambda nd: nd.getValue()

expand = lambda nd: [reconstitute[n.nodeType()](n) for n in nd.getValue()]

def glob(query, lexicon): # lexicon is index.lexicon
    try:
        tree = queryparser.QueryParser(lexicon).parseQuery(query)
    except parsetree.ParseError:
        return None
    if tree is not None:
        return reconstitute[tree.nodeType()](tree)
    else:
        return None



##############################################################################
#
# Copyright (c) 2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Functional tests for `zc.catalog.browser`.

$Id: tests.py 95941 2009-02-01 19:36:08Z nadako $
"""
import doctest
import os.path
import unittest

import transaction

import zope.app.appsetup.bootstrap
import zope.app.appsetup.interfaces
import zope.app.testing.functional
import zope.intid
import zope.intid.interfaces


here = os.path.dirname(os.path.realpath(__file__))

ZcCatalogLayer = zope.app.testing.functional.ZCMLLayer(
    os.path.join(here, "ftesting.zcml"), __name__, "ZcCatalogLayer")


@zope.component.adapter(
    zope.app.appsetup.interfaces.IDatabaseOpenedWithRootEvent)
#
def initializeIntIds(event):
    db, connection, root, root_folder = (
        zope.app.appsetup.bootstrap.getInformationFromEvent(event))
    sm = root_folder.getSiteManager()
    intids = zope.intid.IntIds()
    sm["default"]["test-int-ids"] = intids
    sm.registerUtility(
        intids,
        zope.intid.interfaces.IIntIds)
    transaction.commit()
    connection.close()


def test_suite():
    suite = zope.app.testing.functional.FunctionalDocFileSuite(
        "README.txt",
        optionflags=doctest.ELLIPSIS|doctest.NORMALIZE_WHITESPACE)
    suite.layer = ZcCatalogLayer
    return suite


if __name__ == "__main__":
    unittest.main(defaultTest="test_suite")
